#
# The MIT License
#
# Copyright 2009 Canonical Ltd.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
"""Unit tests for OAuth Server"""
import time
import unittest

from poauth import oauth
from poauth.client import OAuthRequest
from poauth.server import OAuthServer, OAuthStorage, OAuthResponse

class FakeOAuthStorage(OAuthStorage):
    '''Fake storage for testing.'''

    def __init__(self):
        '''Initialize the fake store.'''
        self.consumer = oauth.OAuthConsumer('consumer_key', 'consumer_secret')
        self.request_token = oauth.OAuthToken('request_key', 'request_secret')
        self.access_token = oauth.OAuthToken('access_key', 'access_secret')
        self.verifier = 'verifier'
        self.nonce = 'nonce'
        self.bad_request = OAuthResponse(status=400, message="Bad Request")
        self.unauthorized = OAuthResponse()

    def lookup_consumer(self, consumer_key):
        '''Validate the consumer.'''
        if consumer_key == self.consumer.key:
            return self.consumer
        return None

    def lookup_token(self, token_key):
        '''Look up an existing token.'''
        if token_key == self.request_token.key:
            return oauth.OAuthToken(self.request_token.key,
                                    self.request_token.secret,
                                    verifier=self.verifier)

    def fetch_request_token(self, parameters):
        '''Get a request token.'''
        response_params = {'oauth_token' : self.request_token.key,
                           'oauth_token_secret' : self.request_token.secret,
                           }

        if parameters['oauth_nonce'] != self.nonce:
            return self.bad_request

        consumer = self.lookup_consumer(parameters['oauth_consumer_key'])
        if consumer:
            return OAuthResponse(status=200, parameters=response_params)
        return self.unauthorized

    def authorize_request_token(self, token, parameters):
        '''Authorize the token.'''
        response_params = {'oauth_token' : self.request_token.key,
                           'oauth_verifier' : self.verifier,
                           }
        if parameters['oauth_token'] == self.request_token.key:
            return OAuthResponse(status=200, parameters=response_params)
        return self.unauthorized

    def fetch_access_token(self, token, parameters):
        '''Fetch the access token.'''
        response_params = {'oauth_token' : self.access_token.key,
                           'oauth_token_secret' : self.access_token.secret,
                           }
        verifier = parameters.get('oauth_verifier', self.verifier)
        if verifier == self.verifier:
            return OAuthResponse(status=200, parameters=response_params)
        return self.unauthorized


class OAuthServerTests(unittest.TestCase):
    '''Test the OAuthServer class.'''

    def setUp(self):
        '''Set up needed data for tests.'''
        sig_methods = [oauth.OAuthSignatureMethod_PLAINTEXT(),
                       oauth.OAuthSignatureMethod_HMAC_SHA1(),
                       ]
        self.consumer = oauth.OAuthConsumer('consumer_key', 'consumer_secret')
        self.server = OAuthServer(storage=FakeOAuthStorage(),
                                  realm="https://localhost",
                                  supported_sig_methods=sig_methods)

    def test_get_request_token(self):
        '''Test the creation of a request token.'''
        request_params = {'oauth_timestamp' : str(int(time.time())),
                          'oauth_nonce' : 'nonce',
                          'oauth_version' : oauth.OAUTH_VERSION,
                          }
        request = OAuthRequest(callback="oob", consumer=self.consumer,
                               parameters=request_params)
        method = oauth.OAuthSignatureMethod_PLAINTEXT()
        request.sign_request(method, self.consumer, None)
        response = self.server.get_request_token(request)
        self.assertTrue(isinstance(response, OAuthResponse))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.parameters['oauth_callback_confirmed'],
                          'true')

    def test_authorize_request_token(self):
        '''Test authorizing the request token.'''
        token = oauth.OAuthToken('request_key', 'request_secret')
        request = OAuthRequest(token=token)
        response = self.server.authorize_request_token(request)
        self.assertTrue(isinstance(response, OAuthResponse))
        self.assertEquals(response.status_code, 200)

    def test_get_access_token(self):
        '''Test getting an access token.'''
        request_params = {'oauth_timestamp' : str(int(time.time())),
                          'oauth_nonce' : 'nonce',
                          'oauth_version' : oauth.OAUTH_VERSION,
                          }
        token = oauth.OAuthToken('request_key', 'request_secret',
                                 verifier='verifier')
        request=OAuthRequest(consumer=self.consumer, token=token,
                             parameters=request_params)
        method = oauth.OAuthSignatureMethod_PLAINTEXT()
        request.sign_request(method, self.consumer, token)
        response = self.server.get_access_token(request)
        self.assertTrue(isinstance(response, OAuthResponse))
        self.assertEquals(response.status_code, 200)
        self.assertEquals(response.token.key, 'access_key')
        self.assertEquals(response.token.secret, 'access_secret')

class OAuthStorageTests(unittest.TestCase):
    '''Test the OAuthStorage class.'''

    def setUp(self):
        '''Set up needed data for tests.'''
        self.storage = FakeOAuthStorage()

    def test_validate_consumer(self):
        '''Test consumer validation.'''
        consumer = self.storage.lookup_consumer('consumer_key')
        self.assertTrue(isinstance(consumer, oauth.OAuthConsumer))
        self.assertEquals('consumer_key', consumer.key)

    def test_fetch_request_token(self):
        '''Test fetching the request token.'''
        parameters = {'oauth_consumer_key' : 'consumer_key',
                      'oauth_signature_method' : 'PLAINTEXT',
                      'oauth_signature' : '',
                      'oauth_timestamp' : str(int(time.time())),
                      'oauth_nonce' : 'nonce',
                      'oauth_version' : oauth.OAUTH_VERSION,
                      'oauth_callback' : 'oob',
                      }
        response = self.storage.fetch_request_token(parameters)
        self.assertTrue(isinstance(response, OAuthResponse))
        self.assertEquals(response.token.key, 'request_key')

    def test_fetch_request_token_compat(self):
        '''Test fetching the request token.'''
        parameters = {'oauth_consumer_key' : 'consumer_key',
                      'oauth_signature_method' : 'PLAINTEXT',
                      'oauth_signature' : '',
                      'oauth_timestamp' : str(int(time.time())),
                      'oauth_nonce' : 'nonce',
                      'oauth_version' : oauth.OAUTH_VERSION,
                      }
        response = self.storage.fetch_request_token(parameters)
        self.assertTrue(isinstance(response, OAuthResponse))
        self.assertEquals(response.token.key, 'request_key')

    def test_authorize_request_token(self):
        '''Test authorizing a request token.'''

    def test_fetch_access_token(self):
        '''Test fetching the access token.'''
